﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DotSpatial.Data;
using DotSpatial.Topology;

namespace GeostatisticalTool.Classes
{
    public class DataSourceInterpolators
    {
        private int npoints1 = 0;
        private int np1 = 0;
        private double[,] cpoints1;
        private List<Kpoint> kpoints= new List<Kpoint>();
        private QuadTree quadTree1;
        private double[] extent1 = { double.MaxValue, double.MaxValue, double.MinValue, double.MinValue };//xmin ymin xmax ymax
        private long[] idExtremPoints;
        private long[] idExtremeZPoints;
        private double[] extentZ1 = { double.MaxValue, double.MinValue };//zmin zmax
        private IFeatureSet data;

        //public IFeatureSet Data
        //{
        //    get { return data; }
        //    set { data = value; }
        //}

        private SearchData search;


        public DataSourceInterpolators(IFeatureSet data, int idField)
        {
            this.CreatePointsK(data, idField);
            search = new SearchData(kpoints,quadTree1,extent1);
        }

        #region propierties

        public List<Kpoint> Kpoints
        {
            get { return kpoints; }
            set { kpoints = value; }
        }

        public SearchData Search
        {
            get { return search; }
            set { search = value; }
        }



        public double[] extent
        {
            get { return extent1; }
            set { extent1 = value; }
        }


        public double[] extentZ
        {
            get { return extentZ1; }
            set { extentZ1 = value; }
        }


        public QuadTree quadTree
        {
            get { return quadTree1; }
            set { quadTree1 = value; }
        }
        public double[,] cpoints
        {
            get { return cpoints1; }
            set { cpoints1 = value; }
        }


        /// <summary>Gets a value indicating number of point in the Shapefile</summary> 
        public int npoints
        {
            get { return this.npoints1; }
        }

        /// <summary>Gets a value indicating number of points used by search</summary> 
        public int np
        {
            get { return this.np1; }
            set { this.np1=value; }
        }

        #endregion 

        public SearchData GetSearch()
        {
            return this.search;
        }
        
        private void CreatePoints(
           IFeatureSet shapeLayer, int idField)
        {
            this.npoints1 = shapeLayer.NumRows();
            this.cpoints1 = new double[this.npoints1, 6];//x,y,z,z-inter,error,stderr
            this.quadTree1 = new QuadTree();

            for (int shp = 0; shp < shapeLayer.NumRows(); shp++)
            {
                Coordinate pt = shapeLayer.Features[shp].Coordinates[0];
                this.cpoints1[shp, 0] = pt.X;
                this.cpoints1[shp, 1] = pt.Y;
                this.cpoints1[shp, 2] = Convert.ToDouble(shapeLayer.Features[shp].DataRow[idField]);
               // this.quadTree1.Insert(pt.X, pt.Y, shp);



                if (this.cpoints1[shp, 2] > this.extentZ1[1])
                {
                    this.extentZ1[1] = this.cpoints1[shp, 2];
                }
                if (this.cpoints1[shp, 2] < this.extentZ1[0])
                {
                    this.extentZ1[0] = this.cpoints1[shp, 2];
                }

                if (pt.Y > this.extent1[3])
                {
                    this.extent1[3] = pt.Y;
                }

                if (pt.X > this.extent1[2])
                {
                    this.extent1[2] = pt.X;
                }

                if (pt.X < this.extent1[0])
                {
                    this.extent1[0] = pt.X;
                }

                if (pt.Y < this.extent1[1])
                {
                    this.extent1[1] = pt.Y;
                }
            }
            

        }


        private void CreatePointsK(
          IFeatureSet shapeLayer, int idField)
        {
            this.data = shapeLayer;
            this.npoints1 = shapeLayer.NumRows();
           // this.kpoints = new Kpoint[this.npoints1];//x,y,z,z-inter,error,stderr
            this.quadTree1 = new QuadTree();
            Stat NorthSouth = new Stat(false);
            Stat EastWest= new Stat(false);
            Stat Zvalue = new Stat(false);

            for (int shp = 0; shp < shapeLayer.NumRows(); shp++)
            {
                Coordinate pt = shapeLayer.Features[shp].Coordinates[0];
                Kpoint point = new Kpoint(pt.X, pt.Y, Convert.ToDouble(shapeLayer.Features[shp].DataRow[idField]), shp);
                this.kpoints.Add(point );
                //this.cpoints1[shp, 0] = pt.X;
                //this.cpoints1[shp, 1] = pt.Y;
                //this.cpoints1[shp, 2] = Convert.ToDouble(shapeLayer.Features[shp].DataRow[idField]);
                this.quadTree1.Insert(pt.X, pt.Y, point);
                NorthSouth += new Stat(pt.Y);
                EastWest += new Stat(pt.X);
                Zvalue +=  new Stat(this.kpoints[shp].Z);
            }
                this.extentZ1= new  double[2] {Zvalue.Min,Zvalue.Max};
                this.extent1= new  double[4]{EastWest.Min,NorthSouth.Min,EastWest.Max,NorthSouth.Max};
                this.idExtremPoints= new long[4]{EastWest.PosMin,NorthSouth.PosMin,EastWest.PosMax,NorthSouth.PosMax};
                this.idExtremeZPoints= new long[2]{Zvalue.PosMin,Zvalue.PosMax};
        }

    }
}
